#!/usr/bin/env bash

source /etc/powerdns/blocklists.conf

BLOCKLIST_FILE="/var/lib/powerdns/recursor/blocklist.lua"

ALL_DOMAINS_UNSORTED=$(mktemp)

for i in ${SOURCES}; do
	TYPE=$(echo "${i}" | cut -d '#' -f 1)
	URL=$(echo "${i}" | cut -d '#' -f 2)

	RAW_RULES_FILE=$(mktemp)
	echo "Fetching rules from ${URL}..."
	curl -sL ${URL} -o "${RAW_RULES_FILE}"
	CURL_RET=${?}
	DOMAINS_FILE=$(mktemp)
	case "${TYPE}" in
		ublock)
			echo Extracting domains from ublock-style rules...
			grep -E '^\|\|([a-z0-9]+(-[a-z0-9]+)*\.)+[a-z]{2,}\^$' "${RAW_RULES_FILE}" | tr -d '|^' >>"${DOMAINS_FILE}"
			;;
		raw)
			echo Filtering raw domains...
			grep -E '^([a-z0-9]+(-[a-z0-9]+)*\.)+[a-z]{2,}$' "${RAW_RULES_FILE}" >>"${DOMAINS_FILE}"
			;;
	esac
	rm -f "${RAW_RULES_FILE}"
	DOMAINS_COUNT=$(wc -l "${DOMAINS_FILE}" | cut -d ' ' -f 1)
	if [[ "${CURL_RET}" == "0" ]] && [[ ${DOMAINS_COUNT} -gt 0 ]]; then
		echo "Fetched domains: ${DOMAINS_COUNT}"
		cat "${DOMAINS_FILE}" >>${ALL_DOMAINS_UNSORTED}
	else
		echo "No rules fetched"
	fi
	rm -f "${DOMAINS_FILE}"
done

echo "Total rules: $(wc -l ${ALL_DOMAINS_UNSORTED} | cut -d ' ' -f 1)"

echo "Sorting domains and removing duplicates..."
ALL_DOMAINS_SORTED=$(mktemp)
sort -u "${ALL_DOMAINS_UNSORTED}" >"${ALL_DOMAINS_SORTED}"
rm -f "${ALL_DOMAINS_UNSORTED}"

echo "Unique domains: $(wc -l ${ALL_DOMAINS_SORTED} | cut -d ' ' -f 1)"

echo "Composing Lua script for PowerDNS Recursor..."
awk 'BEGIN{printf("return{")}{printf("\"%s\",",$0)}END{printf("}")}' "${ALL_DOMAINS_SORTED}" >"${BLOCKLIST_FILE}"
rm -f "${ALL_DOMAINS_SORTED}"
echo "Size in bytes: $(stat -c '%s' "${BLOCKLIST_FILE}")"

echo "Reloading PowerDNS Recursor..."

rec_control --timeout=60 reload-lua-script

echo "Done."
